using Autofac;
using FitConnect;
using FitConnect.Interfaces;
using FitConnect.Interfaces.Subscriber;
using FitConnect.Models;
using FluentAssertions;
using Microsoft.Extensions.Logging;

namespace ValidationTests;

public class E2ETestsWithSchemaValidation {
    private ISender _sender;
    private ISubscriber _subscriber;
    private ILogger _logger;
    private IFitConnectSettings _settings;

    public E2ETestsWithSchemaValidation() {
        _logger = LoggerFactory.Create(c => c.AddSimpleConsole()).CreateLogger("Test");

        _settings = MockContainer.Container.Create().Resolve<IFitConnectSettings>();
        _sender = ClientFactory.GetSenderClient(FitConnectEnvironment.Test,
            _settings.SenderClientId, _settings.SenderClientSecret,
            _logger);

        _subscriber = ClientFactory.GetSubscriberClient(FitConnectEnvironment.Test,
            _settings.SubscriberClientId, _settings.SubscriberClientSecret,
            _settings.PrivateDecryptionKeys,
            _settings.PrivateSigningKey,
            _logger);
    }

    [Fact]
    public async void Transmission_WithoutSchemaValidation_ShouldPass() {
        var submission = SendableSubmission.Builder()
            .SetDestination(_settings.DestinationId)
            .SetServiceType(_settings.ServiceIdentifier, "Simple Transmission")
            .SetJsonData(@"{""name"":""John Doe"", ""age"":42}",
                _settings.JsonSchemaUri)
            .Build();

        var sentSubmission = await _sender.SendAsync(submission);

        (await _sender.GetStatusForSubmissionAsync(sentSubmission)).SubmissionState
            .Should().BeOneOf(new List<EventType>() { EventType.Submit, EventType.Notify });

        var onlineSubmissions =
            await _subscriber.GetAvailableSubmissionsAsync(_settings.DestinationId);
        foreach (var submissionForPickup in onlineSubmissions.Where(s =>
                     s.SubmissionId == sentSubmission.SubmissionId)) {
            var pickedUpSubmission = await _subscriber.RequestSubmissionAsync(submissionForPickup);
            pickedUpSubmission.Should().NotBeNull();
            pickedUpSubmission!.GetData().Should()
                .Be(@"{""name"":""John Doe"", ""age"":42}");

            await pickedUpSubmission.AcceptSubmissionAsync();
        }

        (await _sender.GetStatusForSubmissionAsync(sentSubmission)).SubmissionState.Should()
            .Be(EventType.Accept);
    }
}
